@@ -2,10 +2,13 @@
 
 #define debug 0
 
+extern XImage *bimg;
+
 extern char *SEIKEI_LEVEL;
 extern int CHAR_HEADER;
 extern int CHAR_FOOTER;
-extern BOUTEN_T bouten_type[8];
+extern BOUTEN_T bouten_type[];
+extern BOUTEN_T bousen_type[];
 
 static int CHU_NO = 0;
 static int RUBY_YPOS = 0;
@@ -279,6 +282,86 @@ static int XtextDrawRubiString(	Display *disp,
 
 	return 0;
 }
+static int xtextDrawArc(	Display *disp,
+				my_window_t *win,
+				int x0,
+				int y0,
+				int x1,
+				int y1,
+				int type		)
+{
+	XPoint point[4];
+	int w,h;
+
+	switch(type){
+		case 0:case 1:
+			if(bimg != NULL)
+				XPutImage(disp,win->win,win->gc,bimg,0,0,x0,y0+(y1-y0)/2,bimg->width,bimg->height);
+		break;
+		case 2:
+			w = (x1 - x0) - 3;
+			h = w;
+			XFillArc(disp,win->win,win->gc,x0,y0+((y1-y0)-h)/2,w,h,0,360*64);
+		break;
+		case 3:
+			w = (x1 - x0) - 3;
+			h = w;
+			XDrawArc(disp,win->win,win->gc,x0,y0+((y1-y0)-h)/2,w,h,0,360*64);
+		break;
+		case 4:
+			w = (x1 - x0);
+			h = w;
+
+			point[0].x = x0 + w/2;
+			point[0].y = y0 + h/4;
+			point[1].x = x0;
+			point[1].y = y1 - h/4;
+			point[2].x = x1;
+			point[2].y = y1 - h/4;
+			point[3].x = x0 + w/2;
+			point[3].y = y0 + h/4;
+
+			XFillPolygon(disp,win->win,win->gc,point,4,Complex,CoordModeOrigin);
+
+		break;
+		case 5:
+			w = (x1 - x0);
+			h = w;
+
+			point[0].x = x0 + w/2;
+			point[0].y = y0 + h/4;
+			point[1].x = x0;
+			point[1].y = y1 - h/4;
+			point[2].x = x1;
+			point[2].y = y1 - h/4;
+			point[3].x = x0 + w/2;
+			point[3].y = y0 + h/4;
+
+			XDrawLines(disp,win->win,win->gc,point,4,CoordModeOrigin);
+		break;
+		case 6:
+			w = (x1 - x0) - 1;
+			h = w;
+			XDrawArc(disp,win->win,win->gc,x0,y0+((y1-y0)-h)/2,w,h,0,360*64);
+
+			w = (x1 - x0) - 5;
+			h = w;
+			XDrawArc(disp,win->win,win->gc,x0 + 2,y0+((y1-y0)-h)/2,w,h,0,360*64);
+		break;
+		case 7:
+			w = (x1 - x0) - 1;
+			h = w;
+			XDrawArc(disp,win->win,win->gc,x0,y0+((y1-y0)-h)/2,w,h,0,360*64);
+
+			w = (x1 - x0) - 5;
+			h = w;
+			XFillArc(disp,win->win,win->gc,x0 + 2,y0+((y1-y0)-h)/2,w,h,0,360*64);
+		break;
+
+	}
+
+	return 0;
+}
 static int XtextDrawBoutenString(	Display *disp,
 					my_window_t *win,
 					char_data_t *char_d,
@@ -286,21 +369,114 @@ static int XtextDrawBoutenString(	Display *disp,
 					int x0,
 					int y0			)
 {
-	char bouten[3];
-	int x,y,ypad;
+	int x,y,x1,y1;
 
-	if(win->rubi_fs == NULL) return -1;
+	if(win == NULL) return -1;
+	if(char_d == NULL) return -1;
 
 
-	bouten[0] = bouten_type[btype].c1;
-	bouten[1] = bouten_type[btype].c2;
-	bouten[2] = '\0';
-
-	ypad = (char_d->charSize - char_d->charSize_r) / 2;
-
 	x = x0 + char_d->charSize;
-	y = y0 - ypad + 2;
-	XmbDrawString(disp,win->win,win->rubi_fs,win->sgc,x,y,bouten,2);
+	x1 = x + char_d->charSize_r;
+	y = y0 - char_d->charSize;
+	y1 = y0;
+
+	xtextDrawArc(disp,win,x,y,x1,y1,btype);
+
+	return 0;
+}
+static int xtextDrawLine(	Display *disp,
+				my_window_t *win,
+				int x0,
+				int y0,
+				int x1,
+				int y1,
+				int type		)
+{
+	XSegment seg[2]; 
+	XArc arc[10];
+	int i,p,w,h;
+
+	if(win == NULL) return -1;
+
+	switch(type){
+		case 0:
+			p = 2;
+
+			seg[0].x1 = x0;
+			seg[0].y1 = y0;
+			seg[0].x2 = x1;
+			seg[0].y2 = y1;
+
+		       	seg[1].x1 = x0+p;
+			seg[1].y1 = y0;
+			seg[1].x2 = x1+p;
+			seg[1].y2 = y1;
+
+			XDrawSegments(disp,win->win,win->gc,seg,2);
+		break;
+		case 1:
+			p = y0;
+			for(i=0;;i++){
+				if((p += 2) >= y1) break;
+				if((i % 2) == 0){
+					XDrawLine(disp,win->win,win->gc,x0,p,x1,p+2);
+				}
+			}
+		break;
+		case 2:
+			p = y0;
+			for(i=0;;i++){
+				if((p += 4) >= y1) break;
+				if((i % 2) == 0){
+					XDrawLine(disp,win->win,win->gc,x0,p,x1,p+4);
+				}
+			}
+		break;
+		case 3:
+			w = 2;
+			h = 4;
+
+			for(i=0;;i++){
+				if(y1 <= (y0 + (i * h))) break;
+				arc[i].x = x0;
+				arc[i].y = y0 + (i * h);
+				arc[i].width = w;
+				arc[i].height = h;
+				if(i % 2 == 0){
+					arc[i].angle1 = 90 * 64;
+					arc[i].angle2 = 180 * 64;
+				}else{
+					arc[i].angle1 = 270 * 64;
+					arc[i].angle2 = 180 * 64;
+				}
+			}
+			XDrawArcs(disp,win->win,win->gc,arc,i);
+		break;
+		default:
+			XDrawLine(disp,win->win,win->gc,x0,y0,x1,y1);
+		break;
+	}
+
+	return 0;
+}
+static int XtextDrawBousenString(	Display *disp,
+					my_window_t *win,
+					char_data_t *char_d,
+					int btype,
+					int x0,
+					int y0			)
+{
+	int x,y,x1,y1;
+
+	if(win == NULL) return -1;
+	if(char_d == NULL) return -1;
+
+	x = x0 + char_d->charSize + 1;
+	x1 = x;
+	y = y0 - char_d->charSize + 2;
+	y1 = y0 + 2;
+
+	xtextDrawLine(disp,win,x,y,x1,y1,btype);
 
 	return 0;
 }
@@ -324,6 +500,7 @@ static int _Draw_String(	Display *disp,
 			if(win->page_on){
 
 				if(it->bouten >= 0) XtextDrawBoutenString(disp,win,char_d,it->bouten,x,y);
+				if(it->bousen >= 0) XtextDrawBousenString(disp,win,char_d,it->bousen,x,y);
 
 				if(it->rubi_head != NULL) XtextDrawRubiString(disp,win,char_d,it,x,y);
 			}
