@@ -369,14 +369,12 @@ fprintf(stderr,"main_bookname : %s %d\n",book_name,n);
 
 	NPC = n;
 
-	if(line_head != NULL) ibuf_data_free(line_head,line_tail);
+	if(line_head != NULL) ibuf_data_free(&line_head,&line_tail);
 	if(char_d.book != NULL) free(char_d.book);
 
 	con = iconv_open("UTF-8","EUC-JP");
 	char_d.book = (book_data_t *)malloc(sizeof(book_data_t));
 
-	line_head = NULL;
-	line_tail = NULL;
 	if(itextGetTextData(book_name,title,&line_head,&line_tail,xconf.ruby_on)<0) return -1;
 	xtext_set_offset(line_head,line_tail,off_head);
 
@@ -419,6 +417,7 @@ int _select_book_data(char *book_dir)
 fprintf(stderr,"main_selectdir : %s\n",book_dir);
 #endif
 	xtextBookData2Sub(&book_head,&book_tail,book_dir);
+
 	if(book_head == NULL){
 		xtextSubWindow_Message(disp,main_win,XJP_MESSAGE[1]);
 		return -1;
@@ -427,6 +426,7 @@ fprintf(stderr,"main_selectdir : %s\n",book_dir);
 	if((book_path = (char *)xtextSubWindow(disp,main_win,book_head,book_tail)) == NULL)
 		return -1;
 
+
 	if(strcmp(book_path,XJP_CANCEL) == 0) return -1;
 #if debug
 fprintf(stderr,"main_selectpath : %s\n",book_dir);
