@@ -21,14 +21,20 @@ static sub_window_t * _subwindow_data_alloc()
 
 	return sw;
 }
-static void * _subwindow_data_free(sub_window_t **head,sub_window_t **tail)
+void * _subwindow_data_free(sub_window_t **head,sub_window_t **tail)
 {
 	sub_window_t *sw,*sww;
 
+	if(*head == NULL) return NULL;
+
 	for(sw = *head;sw!=NULL;sw=sww){
 		sww = sw->next;
 
+		/*
 		if(sw->re_str != NULL) free(sw->re_str);
+		if(sw->func_data0 != NULL) free(sw->func_data0);
+		if(sw->line_data != NULL) free(sw->line_data);
+		*/
 		free(sw);
 	}
 
