@@ -13,12 +13,9 @@ static void * _page_data_free(page_data_t **start,page_data_t **end)
 {
 	page_data_t *pt,*ptt;
 
-	for(pt = *start;pt!=NULL;pt=pt->next){
-
+	for(pt = *start;pt!=NULL;pt=ptt){
 		ptt = pt->next;
-
 		free(pt);
-
 	}
 
 	*start = NULL;
@@ -79,10 +76,10 @@ static void * _image_data_free(image_data_t **head,image_data_t **tail)
 {
 	image_data_t *im,*imm;
 
-	for(im = *head;im!=NULL;im=im->next){
+	for(im = *head;im!=NULL;im=imm){
 		imm = im->next;
-
 		XDestroyImage(im->char_image);
+		free(im);
 	}
 
 	*head = NULL;
